@@ -26,21 +26,26 @@
 """
 
 import os
+from pathlib import Path
 from collections import OrderedDict
 from collections.abc import Mapping
 import functools
 import logging
 
-from jsonschema.validators import Draft4Validator
+from jsonschema.validators import validator_for
 
+from .kasusererror import KasUserError
 from . import __file_version__, __compatible_file_version__
 from . import CONFIGSCHEMA
 
 __license__ = 'MIT'
 __copyright__ = 'Copyright (c) Siemens AG, 2017-2021'
 
+SOURCE_DIR_OVERRIDE_KEY = '_source_dir'
+SOURCE_DIR_HOST_OVERRIDE_KEY = '_source_dir_host'
 
-class LoadConfigException(Exception):
+
+class LoadConfigException(KasUserError):
     """
         Class for exceptions that appear while loading a configuration file.
     """
@@ -66,7 +71,8 @@ def load_config(filename):
         raise LoadConfigException('Config file extension not recognized',
                                   filename)
 
-    validator = Draft4Validator(CONFIGSCHEMA)
+    validator_class = validator_for(CONFIGSCHEMA)
+    validator = validator_class(CONFIGSCHEMA)
     validation_error = False
 
     for error in validator.iter_errors(config):
@@ -97,10 +103,10 @@ def load_config(filename):
         logging.warning('Obsolete ''proxy_config'' detected. '
                         'This has no effect and will be rejected soon.')
 
-    return config
+    return (config, config.get(SOURCE_DIR_OVERRIDE_KEY, None))
 
 
-class IncludeException(Exception):
+class IncludeException(KasUserError):
     """
         Class for exceptions that appear in the include mechanism.
     """
@@ -121,12 +127,30 @@ class IncludeHandler:
         relative to the repository root path.
 
         The includes are read and merged from the deepest level upwards.
+
+        In case ``ignore_lock`` is ``false``, kas checks if a file
+        ``<file>.lock.<ext>`` exists next to the first entry in
+        ``top_files``. This filename is then appended to the list of
+        ``top_files``.
     """
 
-    def __init__(self, top_files, top_repo_path):
+    def __init__(self, top_files, top_repo_path, use_lock=True):
         self.top_files = top_files
         self.top_repo_path = top_repo_path
 
+        if use_lock:
+            lockfile = self.get_lockfile()
+            if Path(lockfile).exists():
+                logging.debug('includehandler: append lockfile %s', lockfile)
+                self.top_files.append(lockfile)
+
+    def get_lockfile(self):
+        file = Path(self.top_files[0])
+        return file.parent / (file.stem + '.lock' + file.suffix)
+
+    def get_top_repo_path(self):
+        return self.top_repo_path
+
     def get_config(self, repos=None):
         """
         Parameters:
@@ -169,7 +193,16 @@ class IncludeHandler:
 
             missing_repos = []
             configs = []
-            current_config = load_config(filename)
+            try:
+                current_config, src_dir = load_config(filename)
+                # src_dir must only be set by auto-generated config file
+                if src_dir:
+                    self.top_repo_path = src_dir
+                    repo_path = src_dir
+
+            except FileNotFoundError:
+                raise LoadConfigException('Configuration file not found',
+                                          filename)
             if not isinstance(current_config, Mapping):
                 raise IncludeException('Configuration file does not contain a '
                                        'dictionary as base type')
