@@ -34,11 +34,36 @@ import pathlib
 import signal
 from subprocess import Popen, PIPE
 from .context import get_context
+from .kasusererror import KasUserError, CommandExecError
 
 __license__ = 'MIT'
 __copyright__ = 'Copyright (c) Siemens AG, 2017-2018'
 
 
+class InitBuildEnvError(KasUserError):
+    """
+    Error related to the OE / ISAR environment setup scripts
+    """
+    pass
+
+
+class EnvNotValidError(KasUserError):
+    """
+    The caller environment is not suited for the requested operation
+    """
+    pass
+
+
+class TaskExecError(KasUserError):
+    """
+    Similar to :class:`kas.kasusererror.CommandExecError` but for kas
+    internal tasks
+    """
+    def __init__(self, command, ret_code):
+        self.ret_code = ret_code
+        super().__init__('{} failed: error code {}'.format(command, ret_code))
+
+
 class LogOutput:
     """
         Handles the log output of executed applications
@@ -131,6 +156,7 @@ async def run_cmd_async(cmd, cwd, env=None, fail=True, liveupdate=True):
             for line in logo.stderr:
                 msg += line
         logging.error(msg)
+        raise CommandExecError(cmd, ret)
 
     return (ret, ''.join(logo.stdout))
 
@@ -144,7 +170,7 @@ def run_cmd(cmd, cwd, env=None, fail=True, liveupdate=True):
     (ret, output) = loop.run_until_complete(
         run_cmd_async(cmd, cwd, env, fail, liveupdate))
     if ret and fail:
-        sys.exit(ret)
+        raise CommandExecError(cmd, ret)
     return (ret, output)
 
 
@@ -171,11 +197,10 @@ def repos_fetch(repos):
         tasks.append(asyncio.ensure_future(repo.fetch_async()))
 
     loop = asyncio.get_event_loop()
-    loop.run_until_complete(asyncio.wait(tasks))
-
-    for task in tasks:
-        if task.result():
-            sys.exit(task.result())
+    try:
+        loop.run_until_complete(asyncio.gather(*tasks))
+    except CommandExecError as e:
+        raise TaskExecError('fetch repos', e.ret_code)
 
 
 def repos_apply_patches(repos):
@@ -190,11 +215,10 @@ def repos_apply_patches(repos):
         tasks.append(asyncio.ensure_future(repo.apply_patches_async()))
 
     loop = asyncio.get_event_loop()
-    loop.run_until_complete(asyncio.wait(tasks))
-
-    for task in tasks:
-        if task.result():
-            sys.exit(task.result())
+    try:
+        loop.run_until_complete(asyncio.gather(*tasks))
+    except CommandExecError as e:
+        raise TaskExecError('apply patches', e.ret_code)
 
 
 def get_build_environ(build_system):
@@ -217,15 +241,15 @@ def get_build_environ(build_system):
     for (repo, script) in permutations:
         if os.path.exists(repo.path + '/' + script):
             if init_repo:
-                logging.error('Multiple init scripts found (%s vs. %s). ',
-                              repo.name, init_repo.name)
-                logging.error('Resolve ambiguity by removing one of the repos')
-                sys.exit(1)
+                raise InitBuildEnvError(
+                    'Multiple init scripts found ({} vs. {}). '
+                    'Resolve ambiguity by removing one of the repos'
+                    .format(repo.name, init_repo.name))
+
             init_repo = repo
             init_script = script
     if not init_repo:
-        logging.error('Did not find any init-build-env script')
-        sys.exit(1)
+        raise InitBuildEnvError('Did not find any init-build-env script')
 
     with tempfile.TemporaryDirectory() as temp_dir:
         script = """#!/bin/bash
@@ -275,6 +299,9 @@ def get_build_environ(build_system):
         if env_var in os.environ:
             env[env_var] = os.environ[env_var]
 
+    # filter out 'None' values
+    env = {k: v for (k, v) in env.items() if v is not None}
+
     return env
 
 
@@ -385,16 +412,16 @@ def setup_parser_common_args(parser):
                         help='Skip build steps',
                         default=[])
     parser.add_argument('--force-checkout', action='store_true',
-                        help='Always checkout the desired refspec of each '
-                        'repository, discarding any local changes')
+                        help='Always checkout the desired commit/branch of '
+                        'each repository, discarding any local changes')
     parser.add_argument('--update', action='store_true',
                         help='Pull new upstream changes to the desired '
-                        'refspec even if it is already checked out locally')
+                        'branch even if it is already checked out locally')
 
 
 def setup_parser_preserve_env_arg(parser):
     parser.add_argument('-E', '--preserve-env',
-                        help='Keep current user enviornment block',
+                        help='Keep current user environment block',
                         action='store_true')
 
 
@@ -409,9 +436,8 @@ def run_handle_preserve_env_arg(ctx, os, args, SetupHome):
                                 var)
 
         if not os.isatty(sys.stdout.fileno()):
-            logging.error("Error: --preserve-env can only be "
-                          "run from a tty")
-            sys.exit(1)
+            raise EnvNotValidError(
+                '--preserve-env can only be run from a tty')
 
         ctx.environ = os.environ.copy()
 
