@@ -69,15 +69,17 @@
 import logging
 import os
 import pprint
-import sys
 import yaml
-from kconfiglib import Kconfig, Symbol, Choice, expr_value, TYPE_TO_STR, \
-    MENU, COMMENT, STRING, BOOL, INT, HEX, UNKNOWN
+from kconfiglib import Kconfig, Symbol, Choice, KconfigError, \
+    expr_value, TYPE_TO_STR, MENU, COMMENT, STRING, BOOL, INT, HEX, UNKNOWN
 from kas import __version__, __file_version__
 from kas.context import create_global_context
 from kas.config import CONFIG_YAML_FILE
-from kas.includehandler import load_config as load_config_yaml
+from kas.repos import Repo
+from kas.includehandler import load_config as load_config_yaml, \
+    SOURCE_DIR_OVERRIDE_KEY, SOURCE_DIR_HOST_OVERRIDE_KEY
 from kas.plugins.build import Build
+from kas.kasusererror import KasUserError
 
 try:
     from snack import SnackScreen, EntryWindow, ButtonChoiceWindow, \
@@ -89,13 +91,30 @@ except ImportError:
 __license__ = 'MIT'
 __copyright__ = \
     'Copyright (c) 2011-2019, Ulf Magnusson <ulfalizer@gmail.com>\n' \
-    'Copyright (c) Siemens AG, 2021'
+    'Copyright (c) Siemens AG, 2021-2023'
+
+SOURCE_DIR_HOST_ENV_KEY = '_KAS_REPO_DIR_HOST'
+
+
+class VariableTypeError(KasUserError):
+    pass
+
+
+class MissingModuleError(KasUserError):
+    pass
+
+
+class KConfigLoadError(KasUserError):
+    """
+    The KConfig file could not be found or is invalid
+    """
+    pass
 
 
 def check_sym_is_string(sym):
     if sym.type != STRING:
-        logging.error('Variable %s must be of string type', sym.name)
-        sys.exit(1)
+        raise VariableTypeError('Variable {} must be of string type'
+                                .format(sym.name))
 
 
 def str_representer(dumper, data):
@@ -125,7 +144,7 @@ class Menu:
 
     def load_config(self, filename):
         try:
-            self.orig_config = load_config_yaml(filename)
+            self.orig_config, _ = load_config_yaml(filename)
         except FileNotFoundError:
             self.orig_config = {}
             return
@@ -148,7 +167,7 @@ class Menu:
             else:  # string
                 sym.set_value(symvalue)
 
-    def save_config(self, filename):
+    def save_config(self, filename, top_repo_dir):
         kas_includes = []
         kas_targets = []
         kas_build_system = None
@@ -175,10 +194,9 @@ class Menu:
                 elif sym.type == HEX:
                     menu_configuration[symname] = int(symvalue, 16)
                 else:
-                    logging.error(
-                        'Configuration variable %s uses unsupported type',
-                        symname)
-                    sys.exit(1)
+                    raise VariableTypeError(
+                        'Configuration variable {} uses unsupported type'
+                        .format(symname))
 
             if symname.startswith('KAS_INCLUDE_'):
                 check_sym_is_string(sym)
@@ -200,8 +218,13 @@ class Menu:
                 'version': __file_version__,
                 'includes': kas_includes
             },
-            'menu_configuration': menu_configuration
+            'menu_configuration': menu_configuration,
+            SOURCE_DIR_OVERRIDE_KEY: top_repo_dir
         }
+
+        if SOURCE_DIR_HOST_ENV_KEY in os.environ:
+            config[SOURCE_DIR_HOST_OVERRIDE_KEY] = \
+                os.environ[SOURCE_DIR_HOST_ENV_KEY]
         if kas_build_system:
             config['build_system'] = kas_build_system
         if len(kas_targets) > 0:
@@ -241,14 +264,18 @@ class Menu:
 
     def run(self, args):
         if not newt_available:
-            logging.error(
+            raise MissingModuleError(
                 'Menu plugin requires \'python3-newt\' distribution package.')
-            sys.exit(1)
 
         ctx = create_global_context(args)
 
-        self.kconf = Kconfig(args.kconfig, warn_to_stderr=False)
+        kconfig_file = os.path.abspath(args.kconfig)
+        try:
+            self.kconf = Kconfig(kconfig_file, warn_to_stderr=False)
+        except (KconfigError, FileNotFoundError) as err:
+            raise KConfigLoadError(str(err))
 
+        top_repo_path = Repo.get_root_path(os.path.dirname(kconfig_file))
         config_filename = os.path.join(ctx.kas_work_dir, CONFIG_YAML_FILE)
 
         self.load_config(config_filename)
@@ -260,7 +287,7 @@ class Menu:
         if action == 'exit':
             return
 
-        self.save_config(config_filename)
+        self.save_config(config_filename, top_repo_path)
         self.dump_kconf_warnings()
 
         if action == 'build':
