@@ -28,23 +28,61 @@ import os
 import sys
 import logging
 from urllib.parse import urlparse
+from tempfile import TemporaryDirectory
 from .context import get_context
 from .libkas import run_cmd_async, run_cmd
+from .kasusererror import KasUserError
 
 __license__ = 'MIT'
 __copyright__ = 'Copyright (c) Siemens AG, 2017-2018'
 
 
+class UnsupportedRepoTypeError(KasUserError, NotImplementedError):
+    """
+    The requested repo type is unsupported / not implemented
+    """
+    pass
+
+
+class RepoRefError(KasUserError):
+    """
+    The requested repo reference is invalid, missing or could not be found
+    """
+    pass
+
+
+class PatchFileNotFound(KasUserError, FileNotFoundError):
+    """
+    The requested patch file was not found
+    """
+    pass
+
+
+class PatchMappingError(KasUserError):
+    """
+    The requested patch can not be related to a repo
+    """
+    pass
+
+
+class PatchApplyError(KasUserError):
+    """
+    The provided patch file could not be applied
+    """
+
+
 class Repo:
     """
         Represents a repository in the kas configuration.
     """
 
-    def __init__(self, name, url, path, refspec, layers, patches,
-                 disable_operations):
+    def __init__(self, name, url, path, commit, branch, refspec, layers,
+                 patches, disable_operations):
         self.name = name
         self.url = url
         self.path = path
+        self.commit = commit
+        self.branch = branch
         self.refspec = refspec
         self._layers = layers
         self._patches = patches
@@ -72,15 +110,34 @@ class Repo:
                 except ValueError:
                     continue
             return self.url
+        elif item == 'revision':
+            if self.commit:
+                return self.commit
+            branch = self.branch or self.refspec
+            if not branch:
+                return None
+            (_, output) = run_cmd(self.resolve_branch_cmd(),
+                                  cwd=self.path, fail=False)
+            if output:
+                return output.strip()
+            return branch
+
         # Default behaviour
         raise AttributeError
 
     def __str__(self):
-        return '%s:%s %s %s' % (self.url, self.refspec,
+        if self.commit and self.branch:
+            return '%s:%s(%s) %s %s' % (self.url, self.commit, self.branch,
+                                        self.path, self._layers)
+        return '%s:%s %s %s' % (self.url,
+                                self.commit or self.branch or self.refspec,
                                 self.path, self._layers)
 
+    __legacy_refspec_warned__ = []
+
     @staticmethod
-    def factory(name, repo_config, repo_defaults, repo_fallback_path):
+    def factory(name, repo_config, repo_defaults, repo_fallback_path,
+                repo_overrides={}):
         """
             Returns a Repo instance depending on params.
         """
@@ -101,20 +158,37 @@ class Repo:
                 'path': patches_dict[p]['path'],
             }
             if this_patch['repo'] is None:
-                logging.error('No repo specified for patch entry "%s" and no '
-                              'default repo specified.', p)
-                sys.exit(1)
+                raise PatchMappingError(
+                    'No repo specified for patch entry "{}" and no '
+                    'default repo specified.'.format(p))
+
             patches.append(this_patch)
 
         url = repo_config.get('url', None)
         name = repo_config.get('name', name)
         typ = repo_config.get('type', 'git')
+        commit = repo_config.get('commit', None)
+        branch = repo_config.get('branch', repo_defaults.get('branch', None))
         refspec = repo_config.get('refspec',
                                   repo_defaults.get('refspec', None))
-        if refspec is None and url is not None:
-            logging.error('No refspec specified for repository "%s". This is '
-                          'only allowed for local repositories.', name)
-            sys.exit(1)
+        if commit is None and branch is None and refspec is None \
+                and url is not None:
+            raise RepoRefError('No commit or branch specified for repository '
+                               '"{}". This is only allowed for local '
+                               'repositories.'.format(name))
+        if refspec is None:
+            commit = repo_overrides.get('commit', commit)
+        else:
+            if name not in Repo.__legacy_refspec_warned__:
+                logging.warning('Using deprecated refspec for repository '
+                                '"%s". You should migrate to commit/branch.',
+                                name)
+                Repo.__legacy_refspec_warned__.append(name)
+            if commit is not None or branch is not None:
+                raise RepoRefError('Unsupported mixture of legacy refspec '
+                                   'and commit/branch for repository "{}"'
+                                   .format(name))
+            refspec = repo_overrides.get('commit', refspec)
         path = repo_config.get('path', None)
         disable_operations = False
 
@@ -135,12 +209,12 @@ class Repo:
             disable_operations = True
 
         if typ == 'git':
-            return GitRepo(name, url, path, refspec, layers, patches,
-                           disable_operations)
+            return GitRepo(name, url, path, commit, branch, refspec, layers,
+                           patches, disable_operations)
         if typ == 'hg':
-            return MercurialRepo(name, url, path, refspec, layers, patches,
-                                 disable_operations)
-        raise NotImplementedError('Repo type "%s" not supported.' % typ)
+            return MercurialRepo(name, url, path, commit, branch, refspec,
+                                 layers, patches, disable_operations)
+        raise UnsupportedRepoTypeError('Repo type "%s" not supported.' % typ)
 
     @staticmethod
     def get_root_path(path, fallback=True):
@@ -169,22 +243,39 @@ class RepoImpl(Repo):
         """
             Starts asynchronous repository fetch.
         """
+
         if self.operations_disabled:
             return 0
 
+        refdir = get_context().kas_repo_ref_dir
+        sdir = os.path.join(refdir, self.qualified_name) if refdir else None
+
+        # fetch to refdir
+        if refdir and not os.path.exists(sdir):
+            os.makedirs(refdir, exist_ok=True)
+            with TemporaryDirectory(prefix=self.qualified_name + '.',
+                                    dir=refdir) as tmpdir:
+                (retc, _) = await run_cmd_async(
+                    self.clone_cmd(tmpdir, createref=True),
+                    cwd=get_context().kas_work_dir)
+
+                logging.debug('Created repo ref for %s', self.qualified_name)
+                try:
+                    os.rename(tmpdir, sdir)
+                    if sys.version_info < (3, 8):
+                        # recreate dir so cleanup handler can delete it
+                        os.makedirs(tmpdir, exist_ok=True)
+                except OSError:
+                    logging.debug('repo %s already cloned by other instance',
+                                  self.qualified_name)
+
         if not os.path.exists(self.path):
             os.makedirs(os.path.dirname(self.path), exist_ok=True)
-            sdir = os.path.join(get_context().kas_repo_ref_dir or '',
-                                self.qualified_name)
-            logging.debug('Looking for repo ref dir in %s', sdir)
-
             (retc, _) = await run_cmd_async(
-                self.clone_cmd(sdir),
+                self.clone_cmd(sdir, createref=False),
                 cwd=get_context().kas_work_dir)
-            if retc == 0:
-                logging.info('Repository %s cloned', self.name)
-            if not self.refspec.startswith('refs/'):
-                return retc
+
+            logging.info('Repository %s cloned', self.name)
 
         # Make sure the remote origin is set to the value
         # in the kas file to avoid surprises
@@ -193,28 +284,30 @@ class RepoImpl(Repo):
                 self.set_remote_url_cmd(),
                 cwd=self.path,
                 liveupdate=False)
-            if retc != 0:
-                return retc
         except NotImplementedError:
             logging.warning('Repo implementation does not support changing '
                             'the remote url.')
 
         # take what came out of clone and stick to that forever
-        if self.refspec is None:
+        if self.commit is None and self.branch is None \
+           and self.refspec is None:
             return 0
 
         if not get_context().update:
-            # Does refspec exist in the current repository?
+            # Do commit/branch/refspec exist in the current repository?
             (retc, output) = await run_cmd_async(self.contains_refspec_cmd(),
                                                  cwd=self.path,
                                                  fail=False,
                                                  liveupdate=False)
             if retc == 0:
                 logging.info('Repository %s already contains %s as %s',
-                             self.name, self.refspec, output.strip())
+                             self.name,
+                             self.commit or self.branch or self.refspec,
+                             output.strip())
                 return retc
 
-        # Try to fetch if refspec is missing or if --update argument was passed
+        # Try to fetch if commit/branch/refspec is missing or if --update
+        # argument was passed
         (retc, output) = await run_cmd_async(self.fetch_cmd(),
                                              cwd=self.path,
                                              fail=False)
@@ -229,7 +322,9 @@ class RepoImpl(Repo):
         """
             Checks out the correct revision of the repo.
         """
-        if self.operations_disabled or self.refspec is None:
+        if self.operations_disabled \
+            or (self.commit is None and self.branch is None
+                and self.refspec is None):
             return
 
         if not get_context().force_checkout:
@@ -241,16 +336,24 @@ class RepoImpl(Repo):
                 logging.warning('Repo %s is dirty - no checkout', self.name)
                 return
 
-        (_, output) = run_cmd(self.resolve_branch_cmd(),
-                              cwd=self.path, fail=False)
-        if output:
-            desired_ref = output.strip()
-            branch = True
+        if self.commit:
+            desired_ref = self.commit
+            is_branch = False
         else:
-            desired_ref = self.refspec
-            branch = False
+            (_, output) = run_cmd(self.resolve_branch_cmd(),
+                                  cwd=self.path, fail=False)
+            if output:
+                desired_ref = output.strip()
+                is_branch = True
+            elif self.branch:
+                raise RepoRefError(
+                    'Branch "{}" cannot be found in repository {}'
+                    .format(self.branch, self.name))
+            else:
+                desired_ref = self.refspec
+                is_branch = False
 
-        run_cmd(self.checkout_cmd(desired_ref, branch), cwd=self.path)
+        run_cmd(self.checkout_cmd(desired_ref, is_branch), cwd=self.path)
 
     async def apply_patches_async(self):
         """
@@ -261,8 +364,6 @@ class RepoImpl(Repo):
 
         (retc, _) = await run_cmd_async(self.prepare_patches_cmd(),
                                         cwd=self.path)
-        if retc:
-            return retc
 
         my_patches = []
 
@@ -271,13 +372,10 @@ class RepoImpl(Repo):
                                                             None)
 
             if not other_repo:
-                logging.error('Could not find referenced repo. '
-                              '(missing repo: %s, repo: %s, '
-                              'patch entry: %s)',
-                              patch['repo'],
-                              self.name,
-                              patch['id'])
-                return 1
+                raise PatchMappingError(
+                    'Could not find referenced repo. '
+                    '(missing repo: {}, repo: {}, patch entry: {})'
+                    .format(patch['repo'], self.name, patch['id']))
 
             path = os.path.join(other_repo.path, patch['path'])
             cmd = []
@@ -294,44 +392,43 @@ class RepoImpl(Repo):
                         if os.path.isfile(p):
                             my_patches.append((p, patch['id']))
                         else:
-                            raise FileNotFoundError(p)
+                            raise PatchFileNotFound(p)
             else:
-                logging.error('Could not find patch. '
-                              '(patch path: %s, repo: %s, patch entry: %s)',
-                              path,
-                              self.name,
-                              patch['id'])
-                return 1
+                raise PatchFileNotFound(
+                    'Could not find patch. '
+                    '(patch path: {}, repo: {}, patch entry: {})'
+                    .format(path, self.name, patch['id']))
 
         for (path, patch_id) in my_patches:
             cmd = self.apply_patches_file_cmd(path)
-            (retc, output) = await run_cmd_async(cmd, cwd=self.path)
+            (retc, output) = await run_cmd_async(
+                cmd, cwd=self.path, fail=False)
             if retc:
-                logging.error('Could not apply patch. Please fix repos and '
-                              'patches. (patch path: %s, repo: %s, patch '
-                              'entry: %s, vcs output: %s)',
-                              path, self.name, patch_id, output)
-                return 1
-            else:
-                logging.info('Patch applied. '
-                             '(patch path: %s, repo: %s, patch entry: %s)',
-                             path, self.name, patch_id)
+                raise PatchApplyError(
+                    'Could not apply patch. Please fix repos and '
+                    'patches. (patch path: {}, repo: {}, patch '
+                    'entry: {}, vcs output: {})'
+                    .format(path, self.name, patch_id, output))
+
+            logging.info('Patch applied. '
+                         '(patch path: %s, repo: %s, patch entry: %s)',
+                         path, self.name, patch_id)
 
             cmd = self.add_cmd()
-            (retc, output) = await run_cmd_async(cmd, cwd=self.path)
+            (retc, output) = await run_cmd_async(
+                cmd, cwd=self.path, fail=False)
             if retc:
-                logging.error('Could not add patched files. '
-                              'repo: %s, vcs output: %s)',
-                              self.name, output)
-                return 1
+                raise PatchApplyError(
+                    'Could not add patched files. repo: {}, vcs output: {})'
+                    .format(self.name, output))
 
             cmd = self.commit_cmd()
-            (retc, output) = await run_cmd_async(cmd, cwd=self.path)
+            (retc, output) = await run_cmd_async(
+                cmd, cwd=self.path, fail=False)
             if retc:
-                logging.error('Could not commit patch changes. '
-                              'repo: %s, vcs output: %s)',
-                              self.name, output)
-                return 1
+                raise PatchApplyError(
+                    'Could not commit patch changes. repo: {}, vcs output: {})'
+                    .format(self.name, output))
 
         return 0
 
@@ -341,17 +438,21 @@ class GitRepo(RepoImpl):
         Provides the git functionality for a Repo.
     """
 
-    def remove_ref_prefix(self, refspec):
+    def remove_ref_prefix(self, branch):
         ref_prefix = 'refs/'
-        return refspec[refspec.startswith(ref_prefix) and len(ref_prefix):]
+        return branch[branch.startswith(ref_prefix) and len(ref_prefix):]
 
     def add_cmd(self):
         return ['git', 'add', '-A']
 
-    def clone_cmd(self, gitsrcdir):
-        cmd = ['git', 'clone', '-q', self.effective_url, self.path]
-        if get_context().kas_repo_ref_dir and os.path.exists(gitsrcdir):
-            cmd.extend(['--reference', gitsrcdir])
+    def clone_cmd(self, srcdir, createref):
+        cmd = ['git', 'clone', '-q']
+        if createref:
+            cmd.extend([self.effective_url, '--bare', srcdir])
+        elif srcdir:
+            cmd.extend([srcdir, '--reference', srcdir, self.path])
+        else:
+            cmd.extend([self.effective_url, self.path])
         return cmd
 
     def commit_cmd(self):
@@ -359,15 +460,19 @@ class GitRepo(RepoImpl):
                 '-m', 'msg']
 
     def contains_refspec_cmd(self):
-        return ['git', 'cat-file', '-t', self.remove_ref_prefix(self.refspec)]
+        branch = self.branch or self.refspec
+        if branch and branch.startswith('refs/'):
+            branch = 'remotes/origin/' + self.remove_ref_prefix(branch)
+        return ['git', 'cat-file', '-t', self.commit or branch]
 
     def fetch_cmd(self):
         cmd = ['git', 'fetch', '-q']
-        if self.refspec.startswith('refs/'):
+        branch = self.branch or self.refspec
+        if branch and branch.startswith('refs/'):
             cmd.extend(['origin',
-                        '+' + self.refspec
+                        '+' + branch
                         + ':refs/remotes/origin/'
-                        + self.remove_ref_prefix(self.refspec)])
+                        + self.remove_ref_prefix(branch)])
 
         return cmd
 
@@ -376,21 +481,25 @@ class GitRepo(RepoImpl):
 
     def resolve_branch_cmd(self):
         return ['git', 'rev-parse', '--verify', '-q',
-                'origin/{refspec}'.
-                format(refspec=self.remove_ref_prefix(self.refspec))]
+                'origin/{branch}'.
+                format(branch=self.remove_ref_prefix(
+                    self.branch or self.refspec))]
 
-    def checkout_cmd(self, desired_ref, branch):
+    def checkout_cmd(self, desired_ref, is_branch):
         cmd = ['git', 'checkout', '-q', self.remove_ref_prefix(desired_ref)]
-        if branch:
-            cmd.extend(['-B', self.remove_ref_prefix(self.refspec)])
+        if is_branch:
+            branch = self.remove_ref_prefix(self.branch or self.refspec)
+            branch = branch[branch.startswith('heads/') and len('heads/'):]
+            cmd.extend(['-B', branch])
         if get_context().force_checkout:
             cmd.append('--force')
         return cmd
 
     def prepare_patches_cmd(self):
+        branch = self.branch or self.refspec
         return ['git', 'checkout', '-q', '-B',
                 'patched-{refspec}'.
-                format(refspec=self.remove_ref_prefix(self.refspec))]
+                format(refspec=self.commit or self.remove_ref_prefix(branch))]
 
     def apply_patches_file_cmd(self, path):
         return ['git', 'apply', '--whitespace=nowarn', path]
@@ -407,14 +516,17 @@ class MercurialRepo(RepoImpl):
     def add_cmd(self):
         return ['hg', 'add']
 
-    def clone_cmd(self, srcdir):
+    def clone_cmd(self, srcdir, createref):
+        # Mercurial does not support repo references (object caches)
+        if createref:
+            return ['true']
         return ['hg', 'clone', self.effective_url, self.path]
 
     def commit_cmd(self):
         return ['hg', 'commit', '--user', 'kas <kas@example.com>', '-m', 'msg']
 
     def contains_refspec_cmd(self):
-        return ['hg', 'log', '-r', self.refspec]
+        return ['hg', 'log', '-r', self.commit or self.branch or self.refspec]
 
     def fetch_cmd(self):
         return ['hg', 'pull']
@@ -423,18 +535,19 @@ class MercurialRepo(RepoImpl):
         return ['hg', 'diff']
 
     def resolve_branch_cmd(self):
-        # We never need to care about creating tracking branches in mercurial
-        return ['false']
+        return ['hg', 'identify', '--id', '-r', self.branch or self.refspec,
+                'default']
 
-    def checkout_cmd(self, desired_ref, branch):
+    def checkout_cmd(self, desired_ref, is_branch):
         cmd = ['hg', 'checkout', desired_ref]
         if get_context().force_checkout:
             cmd.append('--clean')
         return cmd
 
     def prepare_patches_cmd(self):
+        refspec = self.commit or self.branch or self.refspec
         return ['hg', 'branch', '-f',
-                'patched-{refspec}'.format(refspec=self.refspec)]
+                'patched-{refspec}'.format(refspec=refspec)]
 
     def apply_patches_file_cmd(self, path):
         return ['hg', 'import', '--no-commit', path]
