@@ -41,7 +41,8 @@ git commit -m "Release $NEW_VERSION"
 git tag -s -m "Release $NEW_VERSION" "$NEW_VERSION"
 git push --follow-tags
 
-python setup.py sdist upload -r pypi
+python3 setup.py sdist
+twine upload -r pypi "dist/kas-$NEW_VERSION.tar.gz"
 
 authors=$(git shortlog -s "$OLD_VERSION".."$NEW_VERSION" | cut -c8- | paste -s -d, - | sed -e 's/,/, /g')
 highlights=$(sed -e "/$OLD_VERSION/,\$d" CHANGELOG.md)
@@ -67,3 +68,6 @@ https://github.com/orgs/siemens/packages/container/package/kas%2Fkas-isar
 git shortlog "$OLD_VERSION..$NEW_VERSION" >> "$prolog"
 
 thunderbird -compose "subject=[ANNOUNCE] Release $NEW_VERSION,to=kas-devel@googlegroups.com,message=$prolog"
+
+echo "Do not forget to update also the docs (https://readthedocs.org/projects/kas/versions/)! Done? [y/Y]"
+read -r
