@@ -23,12 +23,13 @@
 import os
 import stat
 import shutil
+import pytest
 from kas import kas
+from kas.repos import PatchApplyError, PatchFileNotFound, PatchMappingError
 
 
 def test_patch(changedir, tmpdir):
-    tdir = str(tmpdir.mkdir('test_patch'))
-    shutil.rmtree(tdir, ignore_errors=True)
+    tdir = str(tmpdir / 'test_patch')
     shutil.copytree('tests/test_patch', tdir)
     cwd = os.getcwd()
     os.chdir(tdir)
@@ -41,12 +42,11 @@ def test_patch(changedir, tmpdir):
 
 def test_patch_update(changedir, tmpdir):
     """
-        Test that patches are applied correctly after switching refspec from
+        Test that patches are applied correctly after switching a repo from
         a branch to a commit hash and vice-versa with both git and mercurial
         repositories.
     """
-    tdir = str(tmpdir.mkdir('test_patch_update'))
-    shutil.rmtree(tdir, ignore_errors=True)
+    tdir = str(tmpdir / 'test_patch_update')
     shutil.copytree('tests/test_patch', tdir)
     cwd = os.getcwd()
     os.chdir(tdir)
@@ -55,3 +55,23 @@ def test_patch_update(changedir, tmpdir):
     for f in ['kas/tests/test_patch/hello.sh', 'hello/hello.sh']:
         assert os.stat(f)[stat.ST_MODE] & stat.S_IXUSR
     os.chdir(cwd)
+
+
+def test_invalid_patch(changedir, tmpdir):
+    """
+        Test on common errors when applying patches
+    """
+    tdir = str(tmpdir / 'test_patch_invalid')
+    shutil.copytree('tests/test_patch', tdir)
+    cwd = os.getcwd()
+    os.chdir(tdir)
+
+    with pytest.raises(PatchFileNotFound):
+        kas.kas(['shell', 'test-invalid.yml', '-c', 'true'])
+
+    with pytest.raises(PatchMappingError):
+        kas.kas(['shell', 'test-invalid2.yml', '-c', 'true'])
+
+    with pytest.raises(PatchApplyError):
+        kas.kas(['shell', 'test-invalid3.yml', '-c', 'true'])
+    os.chdir(cwd)
